// Music 256a / CS 476a | fall 2016
// CCRMA, Stanford University
//
// Author: Romain Michon (rmichonATccrmaDOTstanfordDOTedu)
// Description: Simple JUCE AM synthesizer

#ifndef MAINCOMPONENT_H_INCLUDED
#define MAINCOMPONENT_H_INCLUDED

#include "../JuceLibraryCode/JuceHeader.h"
#include "Sine.h"
#include "Smooth.h"


class MainContentComponent :
    public AudioAppComponent,
    private Slider::Listener,
    private ToggleButton::Listener
{
public:
    MainContentComponent()
    : carrierFrequency(440.0),
    index(0.0),
    onOff (0.0)
    {
        // configuring frequency sliders and adding them to the main window
        addAndMakeVisible (carrierSlider);
        carrierSlider.setRange (50.0, 5000.0);
        carrierSlider.setSkewFactorFromMidPoint (500.0);
        carrierSlider.setValue(1000);
        carrierSlider.addListener (this);
        
        // configuring frequency slider and adding it to the main window
        addAndMakeVisible (modulatorSlider);
        modulatorSlider.setRange (50.0, 5000.0);
        modulatorSlider.setSkewFactorFromMidPoint (500.0);
        modulatorSlider.setValue(50);
        modulatorSlider.addListener (this);
        
        // configuring index slider and adding it to the main window
        addAndMakeVisible (indexSlider);
        indexSlider.setRange (0.0, 1.0);
        indexSlider.setValue(0.0);
        indexSlider.addListener (this);
        
        // configuring carrier label box and adding it to the main window
        addAndMakeVisible(carrierLabel);
        carrierLabel.setText ("Carrier Freq", dontSendNotification);
        carrierLabel.attachToComponent (&carrierSlider, true);
        
        // configuring modulator label box and adding it to the main window
        addAndMakeVisible(modulatorLabel);
        modulatorLabel.setText ("Modulator Freq", dontSendNotification);
        modulatorLabel.attachToComponent (&modulatorSlider, true);
       
        // configuring carrier label box and adding it to the main window
        addAndMakeVisible(indexLabel);
        indexLabel.setText ("Index", dontSendNotification);
        indexLabel.attachToComponent (&indexSlider, true);
        
        // configuring gain slider and adding it to the main window
        addAndMakeVisible (gainSlider);
        gainSlider.setRange (0.0, 1.0);
        gainSlider.setValue(0.5);
        gainSlider.addListener (this);
        
        // configuring gain label and adding it to the main window
        addAndMakeVisible(gainLabel);
        gainLabel.setText ("Gain", dontSendNotification);
        gainLabel.attachToComponent (&gainSlider, true);
        
        // configuring on/off button and adding it to the main window
        addAndMakeVisible(onOffButton);
        onOffButton.addListener(this);
        
        // configuring on/off label and adding it to the main window
        addAndMakeVisible(onOffLabel);
        onOffLabel.setText ("On/Off", dontSendNotification);
        onOffLabel.attachToComponent (&onOffButton, true);
        
        setSize (600, 160);
        setAudioChannels (0, 1); // no inputs, one output
    }
    
    ~MainContentComponent()
    {
        shutdownAudio();
    }
    
    void resized() override
    {
        // placing the UI elements in the main window
        // getWidth has to be used in case the window is resized by the user
        const int sliderLeft = 80;
        carrierSlider.setBounds (sliderLeft, 10, getWidth() - sliderLeft - 20, 20);
        modulatorSlider.setBounds (sliderLeft, 40, getWidth() - sliderLeft - 20, 20);
        indexSlider.setBounds (sliderLeft, 70, getWidth() - sliderLeft - 20, 20);
        gainSlider.setBounds (sliderLeft, 100, getWidth() - sliderLeft - 20, 20);
        onOffButton.setBounds (sliderLeft, 130, getWidth() - sliderLeft - 20, 20);
    }
    
    void sliderValueChanged (Slider* slider) override
    {
        if (carrier.getSamplingRate() > 0.0){
            if (slider == &carrierSlider)
            {
                carrierFrequency = carrierSlider.getValue();
            }
            else if (slider == &modulatorSlider)
            {
                modulator.setFrequency(modulatorSlider.getValue());
            }
            else if (slider == &indexSlider)
            {
                index = indexSlider.getValue();
            }
        }
    }
    
    void buttonClicked (Button* button) override
    {
        if(button == &onOffButton && onOffButton.getToggleState()){
            onOff = 1.0;
        }
        else{
            onOff = 0.0;
        }
    }
    
    void prepareToPlay (int /*samplesPerBlockExpected*/, double sampleRate) override
    {
        carrier.setSamplingRate(sampleRate);
        carrier.setFrequency(carrierFrequency);
        modulator.setSamplingRate(sampleRate);
        modulator.setFrequency(50.0);
        for(int i=0; i<4; i++){
            smooth[i].setSmooth(0.999);
        }
    }
    
    void releaseResources() override
    {
    }
    
    void getNextAudioBlock (const AudioSourceChannelInfo& bufferToFill) override
    {
        const float level = gainSlider.getValue(); // gain is updated every block
        // getting the audio output buffer to be filled
        float* const buffer = bufferToFill.buffer->getWritePointer (0, bufferToFill.startSample);
        
        // computing one block
        for (int sample = 0; sample < bufferToFill.numSamples; ++sample)
        {
            carrier.setFrequency(smooth[0].tick(carrierFrequency)); // smoothing freq of the carrier
            buffer[sample] =
                carrier.tick() * // carrier signal
                smooth[1].tick(level) * // applying level
                (1 - modulator.tick()*smooth[2].tick(index)) * // modulation signal
                smooth[3].tick(onOff); // envelope
        }
    }
    
    
private:
    Slider carrierSlider;
    Slider modulatorSlider;
    Slider indexSlider;
    Slider gainSlider;
    ToggleButton onOffButton;
    
    Label carrierLabel;
    Label modulatorLabel;
    Label indexLabel;
    Label gainLabel;
    Label onOffLabel;
    
    Sine carrier, modulator;
    Smooth smooth[4]; // to prevent cliking of some of the parameters
    
    double carrierFrequency, index, onOff;
    
    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MainContentComponent)
};

Component* createMainContentComponent()     { return new MainContentComponent(); }


#endif  // MAINCOMPONENT_H_INCLUDED
